<?php

namespace Drupal\instagram_feeds;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Lists instagram_account entities.
 */
class InstagramAccountListBuilder extends EntityListBuilder {

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * Constructs a new UserListBuilder object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type definition.
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   *   The entity storage class.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   */
  public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage, DateFormatterInterface $date_formatter) {
    parent::__construct($entity_type, $storage);
    $this->dateFormatter = $date_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity.manager')->getStorage($entity_type->id()),
      $container->get('date.formatter')
    );
  }


  /**
   * Get the list of columns to render.
   *
   * @return array
   *   The list of columns, where keys are field names and values are headers.
   */
  protected function getColumns() {
    return [
      'iid' => $this->t('Instagram User ID'),
      'account' => $this->t('Instagram Username'),
      'cron_import_limit' => $this->t('Cron Import Limit'),
      'media_bundle' => $this->t('Media Type'),
      'status' => $this->t('Status'),
      'token_expiration' => $this->t('Token Expiry'),
      'uid' => $this->t('Author'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header = [];
    foreach ($this->getColumns() as $field => $label) {
      $header[$field] = [
        'data' => $label,
        'field' => $field,
        'specifier' => $field,
      ];
    }
    $header['iid']['class'] = [RESPONSIVE_PRIORITY_LOW];
    $header['media_bundle']['class'] = [RESPONSIVE_PRIORITY_LOW];
    $header['cron_import_limit']['class'] = [RESPONSIVE_PRIORITY_LOW];
    $header['token_expiration']['sort'] = 'desc';

    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    $row = [];
    /* @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    foreach (array_keys($this->getColumns()) as $field) {
      $value = $entity->get($field)->first()->getValue();
      $row[$field]['data']['#markup'] = reset($value);
    }
    $row['uid']['data'] = [
      '#theme' => 'username',
      '#account' => $entity->uid->entity,
    ];
    $row['account'] = $entity->toLink();
    $row['status'] = $row['status'] ? $this->t('Enabled') : $this->t('Disabled');
    $row['token_expiration']['data'] = $this->dateFormatter->format($entity->token_expiration->value, 'short');

    return $row + parent::buildRow($entity);
  }

}
